from graphql.execution.base import ResolveInfo

from dagster import RunConfig, check
from dagster.core.definitions import create_environment_schema
from dagster.core.definitions.pipeline import ExecutionSelector, PipelineRunsFilter
from dagster.core.execution.api import create_execution_plan
from dagster.core.types.evaluator import evaluate_config

from .fetch_pipelines import (
    get_dauphin_pipeline_from_selector_or_raise,
    get_dauphin_pipeline_reference_from_selector,
)
from .utils import UserFacingGraphQLError, capture_dauphin_error


def validate_config(graphene_info, dauphin_pipeline, env_config, mode):
    get_validated_config(graphene_info, dauphin_pipeline, env_config, mode)


def get_validated_config(graphene_info, dauphin_pipeline, environment_dict, mode):
    check.str_param(mode, 'mode')

    pipeline = dauphin_pipeline.get_dagster_pipeline()

    environment_schema = create_environment_schema(pipeline, mode)

    validated_config = evaluate_config(
        environment_schema.environment_type, environment_dict, pipeline
    )

    if not validated_config.success:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('PipelineConfigValidationInvalid')(
                pipeline=dauphin_pipeline,
                errors=[
                    graphene_info.schema.type_named(
                        'PipelineConfigValidationError'
                    ).from_dagster_error(graphene_info, err)
                    for err in validated_config.errors
                ],
            )
        )

    return validated_config


def get_run(graphene_info, run_id):
    instance = graphene_info.context.instance
    run = instance.get_run_by_id(run_id)
    if not run:
        return graphene_info.schema.type_named('PipelineRunNotFoundError')(run_id)
    else:
        return graphene_info.schema.type_named('PipelineRun')(run)


def get_run_tags(graphene_info):
    instance = graphene_info.context.instance
    return [
        graphene_info.schema.type_named('PipelineTagAndValues')(key=key, values=values)
        for key, values in instance.get_run_tags()
    ]


def get_runs(graphene_info, filters, cursor=None, limit=None):
    check.inst_param(filters, 'filters', PipelineRunsFilter)
    check.opt_str_param(cursor, 'cursor')
    check.opt_int_param(limit, 'limit')

    instance = graphene_info.context.instance
    runs = []

    if filters.run_id:
        run = instance.get_run_by_id(filters.run_id)
        if run:
            runs = [run]
    elif filters.pipeline:
        runs = instance.get_runs_with_pipeline_name(filters.pipeline, cursor=cursor, limit=limit)
    elif filters.tag_key:
        runs = instance.get_runs_with_matching_tag(
            filters.tag_key, filters.tag_value, cursor=cursor, limit=limit
        )
    elif filters.status:
        runs = instance.get_runs_with_status(filters.status, cursor=cursor, limit=limit)
    else:
        runs = instance.all_runs(cursor=cursor, limit=limit)

    return [graphene_info.schema.type_named('PipelineRun')(run) for run in runs]


@capture_dauphin_error
def validate_pipeline_config(graphene_info, selector, environment_dict, mode):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    check.opt_str_param(mode, 'mode')

    dauphin_pipeline = get_dauphin_pipeline_from_selector_or_raise(graphene_info, selector)
    get_validated_config(graphene_info, dauphin_pipeline, environment_dict, mode)
    return graphene_info.schema.type_named('PipelineConfigValidationValid')(dauphin_pipeline)


@capture_dauphin_error
def get_execution_plan(graphene_info, selector, environment_dict, mode):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    check.opt_str_param(mode, 'mode')

    dauphin_pipeline = get_dauphin_pipeline_reference_from_selector(graphene_info, selector)
    get_validated_config(graphene_info, dauphin_pipeline, environment_dict, mode)
    return graphene_info.schema.type_named('ExecutionPlan')(
        dauphin_pipeline,
        create_execution_plan(
            dauphin_pipeline.get_dagster_pipeline(), environment_dict, RunConfig(mode=mode)
        ),
    )



from dagster_graphql.client.mutations import (
    execute_execute_plan_mutation,
    execute_execute_plan_mutation_raw,
    execute_start_pipeline_execution_mutation,
)

from dagster import ExecutionTargetHandle

EXPECTED_EVENTS = {
    ('ENGINE_EVENT', None),
    ('STEP_INPUT', 'sleeper.compute'),
    ('STEP_INPUT', 'sleeper_2.compute'),
    ('STEP_INPUT', 'sleeper_3.compute'),
    ('STEP_INPUT', 'sleeper_4.compute'),
    ('STEP_INPUT', 'total.compute'),
    ('STEP_OUTPUT', 'giver.compute'),
    ('STEP_OUTPUT', 'sleeper.compute'),
    ('STEP_OUTPUT', 'sleeper_2.compute'),
    ('STEP_OUTPUT', 'sleeper_3.compute'),
    ('STEP_OUTPUT', 'sleeper_4.compute'),
    ('STEP_OUTPUT', 'total.compute'),
    ('STEP_START', 'giver.compute'),
    ('STEP_START', 'sleeper.compute'),
    ('STEP_START', 'sleeper_2.compute'),
    ('STEP_START', 'sleeper_3.compute'),
    ('STEP_START', 'sleeper_4.compute'),
    ('STEP_START', 'total.compute'),
    ('STEP_SUCCESS', 'giver.compute'),
    ('STEP_SUCCESS', 'sleeper.compute'),
    ('STEP_SUCCESS', 'sleeper_2.compute'),
    ('STEP_SUCCESS', 'sleeper_3.compute'),
    ('STEP_SUCCESS', 'sleeper_4.compute'),
    ('STEP_SUCCESS', 'total.compute'),
}


def test_execute_start_pipeline_execution_mutation():
    pipeline_name = 'sleepy_pipeline'
    handle = ExecutionTargetHandle.for_pipeline_module(
        'dagster_examples.toys.sleepy', pipeline_name
    )
    variables = {
        'executionParams': {
            'environmentConfigData': {},
            'mode': 'default',
            'selector': {'name': pipeline_name},
            'executionMetadata': {'runId': '12345'},
        }
    }
    result = execute_start_pipeline_execution_mutation(handle, variables)
    seen_events = set()
    for event in result:
        seen_events.add((event.event_type_value, event.step_key))

    assert seen_events == EXPECTED_EVENTS


def test_execute_execute_plan_mutation():
    pipeline_name = 'sleepy_pipeline'
    handle = ExecutionTargetHandle.for_pipeline_module(
        'dagster_examples.toys.sleepy', pipeline_name
    )
    variables = {
        'executionParams': {
            'environmentConfigData': {},
            'mode': 'default',
            'selector': {'name': pipeline_name},
            'executionMetadata': {'runId': '12345'},
        }
    }
    result = execute_execute_plan_mutation(handle, variables)
    seen_events = set()
    for event in result:
        seen_events.add((event.event_type_value, event.step_key))

    assert seen_events == EXPECTED_EVENTS


def test_execute_execute_plan_mutation_raw():
    pipeline_name = 'sleepy_pipeline'
    handle = ExecutionTargetHandle.for_pipeline_module(
        'dagster_examples.toys.sleepy', pipeline_name
    )
    variables = {
        'executionParams': {
            'environmentConfigData': {},
            'mode': 'default',
            'selector': {'name': pipeline_name},
            'executionMetadata': {'runId': '12345'},
        }
    }
    result = execute_execute_plan_mutation_raw(handle, variables)
    seen_events = set()
    for event in result:
        seen_events.add((event.dagster_event.event_type_value, event.step_key))

    assert seen_events == EXPECTED_EVENTS



from dagster_graphql.test.utils import execute_dagster_graphql

from .setup import define_context

RESOURCE_QUERY = '''
{
  pipeline(params: { name: "multi_mode_with_resources" }) {
    __typename
    ... on Pipeline {
      modes {
        name
        resources {
          name
          description
          configField {
            configType {
              name
              ... on CompositeConfigType {
                fields {
                  name
                  configType {
                    name
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
'''

REQUIRED_RESOURCE_QUERY = '''{
  pipeline(params: { name:"required_resource_pipeline" }){
    name
    solids {
      definition {
          requiredResources {
            resourceKey
          }
      }
    }
  }
}
'''


def test_mode_fetch_resources(snapshot):
    result = execute_dagster_graphql(define_context(), RESOURCE_QUERY)

    assert not result.errors
    assert result.data
    assert result.data['pipeline']
    assert result.data['pipeline']['modes']
    for mode_data in result.data['pipeline']['modes']:
        assert mode_data['resources']

    snapshot.assert_match(result.data)


def test_required_resources(snapshot):
    result = execute_dagster_graphql(define_context(), REQUIRED_RESOURCE_QUERY)

    assert not result.errors
    assert result.data
    assert result.data['pipeline']['solids']
    [solid] = result.data['pipeline']['solids']
    assert solid
    assert solid['definition']['requiredResources']
    assert solid['definition']['requiredResources'] == [{'resourceKey': 'R1'}]

    snapshot.assert_match(result.data)



import copy

from dagster_graphql.test.utils import execute_dagster_graphql

from dagster.core.instance import DagsterInstance

from .utils import define_context, sync_execute_get_run_log_data

RUNS_QUERY = '''
query PipelineRunsRootQuery($name: String!) {
  pipeline(params: { name: $name }) {
    ...on PipelineReference { name }
    ... on Pipeline {
      runs {
        ...RunHistoryRunFragment
      }
    }
  }
}

fragment RunHistoryRunFragment on PipelineRun {
  runId
  status
  pipeline {
    ...on PipelineReference { name }
  }
  logs {
    nodes {
      __typename
      ... on MessageEvent {
        timestamp
      }
    }
  }
  executionPlan {
    steps {
      key
    }
  }
  environmentConfigYaml
  mode
  canCancel
}
'''

DELETE_RUN_MUTATION = '''
mutation DeleteRun($runId: String!) {
  deletePipelineRun(runId: $runId) {
    __typename
    ... on DeletePipelineRunSuccess {
      runId
    }
    ... on PythonError {
      message
    }
  }
}
'''


def _get_runs_data(result, run_id):
    for run_data in result.data['pipeline']['runs']:
        if run_data['runId'] == run_id:
            # so caller can delete keys
            return copy.deepcopy(run_data)

    return None


def test_get_runs_over_graphql():
    payload_one = sync_execute_get_run_log_data(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'add_mode',
                'environmentConfigData': {'resources': {'op': {'config': 2}}},
            }
        }
    )
    run_id_one = payload_one['runId']

    payload_two = sync_execute_get_run_log_data(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'add_mode',
                'environmentConfigData': {'resources': {'op': {'config': 3}}},
            }
        }
    )

    run_id_two = payload_two['runId']

    read_context = define_context(instance=DagsterInstance.local_temp())

    result = execute_dagster_graphql(
        read_context, RUNS_QUERY, variables={'name': 'multi_mode_with_resources'}
    )

    run_one_data = _get_runs_data(result, run_id_one)
    assert [log['__typename'] for log in run_one_data['logs']['nodes']] == [
        msg['__typename'] for msg in payload_one['messages']
    ]

    run_two_data = _get_runs_data(result, run_id_two)
    assert [log['__typename'] for log in run_two_data['logs']['nodes']] == [
        msg['__typename'] for msg in payload_two['messages']
    ]

    # delete the second run
    result = execute_dagster_graphql(
        read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two}
    )
    assert result.data['deletePipelineRun']['__typename'] == 'DeletePipelineRunSuccess'
    assert result.data['deletePipelineRun']['runId'] == run_id_two

    # query it back out
    result = execute_dagster_graphql(
        read_context, RUNS_QUERY, variables={'name': 'multi_mode_with_resources'}
    )

    # first is the same
    run_one_data = _get_runs_data(result, run_id_one)
    assert [log['__typename'] for log in run_one_data['logs']['nodes']] == [
        msg['__typename'] for msg in payload_one['messages']
    ]

    # second is gone
    run_two_data = _get_runs_data(result, run_id_two)
    assert run_two_data is None

    # try to delete the second run again
    execute_dagster_graphql(read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two})

    result = execute_dagster_graphql(
        read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two}
    )
    assert result.data['deletePipelineRun']['__typename'] == 'PipelineRunNotFoundError'



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['test_get_compute_logs_over_graphql 1'] = {
    'stdout': {
        'data': '''HELLO WORLD
'''
    }
}

snapshots['test_compute_logs_subscription_graphql 1'] = [
    {
        'computeLogs': {
            'data': '''HELLO WORLD
'''
        }
    }
]



from .utils import execute_dagster_graphql

PRESETS_QUERY = '''
query PresetsQuery($name: String!) {
  pipeline(params: { name: $name }) {
    name
    presets {
      __typename
      name
      solidSubset
      environmentConfigYaml
      mode
    }
  }
}  
'''


def execute_preset_query(pipeline_name, context):
    return execute_dagster_graphql(context, PRESETS_QUERY, variables={'name': pipeline_name})



import sys

from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.definitions.pipeline import ExecutionSelector
from dagster.core.errors import DagsterInvalidDefinitionError
from dagster.utils.error import serializable_error_info_from_exc_info

from .utils import UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def get_pipeline_or_error(graphene_info, selector):
    '''Returns a DauphinPipelineOrError.'''
    return get_dauphin_pipeline_from_selector_or_raise(graphene_info, selector)


def get_pipeline_or_raise(graphene_info, selector):
    '''Returns a DauphinPipeline or raises a UserFacingGraphQLError if one cannot be retrieved
    from the selector, e.g., the pipeline is not present in the loaded repository.'''
    return get_dauphin_pipeline_from_selector_or_raise(graphene_info, selector)


def get_pipeline_reference_or_raise(graphene_info, selector):
    '''Returns a DauphinPipelineReference or raises a UserFacingGraphQLError if a pipeline
    reference cannot be retrieved from the selector, e.g, a UserFacingGraphQLError that wraps an
    InvalidSubsetError.'''
    return get_dauphin_pipeline_reference_from_selector(graphene_info, selector)


@capture_dauphin_error
def get_pipelines_or_error(graphene_info):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    return _get_pipelines(graphene_info)


def get_pipelines_or_raise(graphene_info):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    return _get_pipelines(graphene_info)


def _get_pipelines(graphene_info):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)

    repository = graphene_info.context.get_repository()

    pipeline_instances = []
    for pipeline_def in repository.get_all_pipelines():
        pipeline_instances.append(graphene_info.schema.type_named('Pipeline')(pipeline_def))
    return graphene_info.schema.type_named('PipelineConnection')(
        nodes=sorted(pipeline_instances, key=lambda pipeline: pipeline.name)
    )


def get_dagster_pipeline_from_selector(graphene_info, selector):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    repository = graphene_info.context.get_repository()
    if not repository.has_pipeline(selector.name):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('PipelineNotFoundError')(pipeline_name=selector.name)
        )

    orig_pipeline = graphene_info.context.get_pipeline(selector.name)
    if selector.solid_subset:
        for solid_name in selector.solid_subset:
            if not orig_pipeline.has_solid_named(solid_name):
                raise UserFacingGraphQLError(
                    graphene_info.schema.type_named('InvalidSubsetError')(
                        message='Solid "{solid_name}" does not exist in "{pipeline_name}"'.format(
                            solid_name=solid_name, pipeline_name=selector.name
                        ),
                        pipeline=graphene_info.schema.type_named('Pipeline')(orig_pipeline),
                    )
                )
    try:
        return orig_pipeline.build_sub_pipeline(selector.solid_subset)
    except DagsterInvalidDefinitionError:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('InvalidSubsetError')(
                message=serializable_error_info_from_exc_info(sys.exc_info()).message,
                pipeline=graphene_info.schema.type_named('Pipeline')(orig_pipeline),
            )
        )


def get_dauphin_pipeline_from_selector_or_raise(graphene_info, selector):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    return graphene_info.schema.type_named('Pipeline')(
        get_dagster_pipeline_from_selector(graphene_info, selector)
    )


def get_dauphin_pipeline_reference_from_selector(graphene_info, selector):
    from ..schema.errors import DauphinPipelineNotFoundError

    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    try:
        return graphene_info.schema.type_named('Pipeline')(
            get_dagster_pipeline_from_selector(graphene_info, selector)
        )

    except UserFacingGraphQLError as exc:
        if isinstance(exc.dauphin_error, DauphinPipelineNotFoundError):
            return graphene_info.schema.type_named('UnknownPipeline')(selector.name)
        raise



import os
import time

from dagster_graphql.test.utils import execute_dagster_graphql

from dagster import execute_pipeline
from dagster.core.instance import DagsterInstance
from dagster.utils import safe_tempfile_path

from .execution_queries import START_PIPELINE_EXECUTION_QUERY
from .setup import define_subprocess_context

RUN_CANCELLATION_QUERY = '''
mutation($runId: String!) {
  cancelPipelineExecution(runId: $runId){
    __typename
    ... on CancelPipelineExecutionSuccess{
      run {
        runId
      }
    }
    ... on CancelPipelineExecutionFailure {
      run {
        runId
      }
      message
    }
    ... on PipelineRunNotFoundError {
      runId
    }
  }
}
'''


def test_basic_cancellation():
    context = define_subprocess_context(DagsterInstance.local_temp())
    with safe_tempfile_path() as path:
        context = define_subprocess_context(DagsterInstance.local_temp())
        result = execute_dagster_graphql(
            context,
            START_PIPELINE_EXECUTION_QUERY,
            variables={
                'executionParams': {
                    'selector': {'name': 'infinite_loop_pipeline'},
                    'mode': 'default',
                    'environmentConfigData': {'solids': {'loop': {'config': {'file': path}}}},
                }
            },
        )

        assert not result.errors
        assert result.data

        # just test existence
        assert (
            result.data['startPipelineExecution']['__typename'] == 'StartPipelineExecutionSuccess'
        )
        run_id = result.data['startPipelineExecution']['run']['runId']

        assert run_id

        # ensure the execution has happened
        while not os.path.exists(path):
            time.sleep(0.1)

        result = execute_dagster_graphql(
            context, RUN_CANCELLATION_QUERY, variables={'runId': run_id}
        )

        assert (
            result.data['cancelPipelineExecution']['__typename'] == 'CancelPipelineExecutionSuccess'
        )


def test_run_not_found():
    context = define_subprocess_context(DagsterInstance.local_temp())
    result = execute_dagster_graphql(context, RUN_CANCELLATION_QUERY, variables={'runId': 'nope'})
    assert result.data['cancelPipelineExecution']['__typename'] == 'PipelineRunNotFoundError'


def test_terminate_failed():
    with safe_tempfile_path() as path:
        context = define_subprocess_context(DagsterInstance.local_temp())
        old_terminate = context.execution_manager.terminate
        context.execution_manager.terminate = lambda _run_id: False
        result = execute_dagster_graphql(
            context,
            START_PIPELINE_EXECUTION_QUERY,
            variables={
                'executionParams': {
                    'selector': {'name': 'infinite_loop_pipeline'},
                    'mode': 'default',
                    'environmentConfigData': {'solids': {'loop': {'config': {'file': path}}}},
                }
            },
        )

        assert not result.errors
        assert result.data

        # just test existence
        assert (
            result.data['startPipelineExecution']['__typename'] == 'StartPipelineExecutionSuccess'
        )
        run_id = result.data['startPipelineExecution']['run']['runId']
        # ensure the execution has happened
        while not os.path.exists(path):
            time.sleep(0.1)

        result = execute_dagster_graphql(
            context, RUN_CANCELLATION_QUERY, variables={'runId': run_id}
        )
        assert (
            result.data['cancelPipelineExecution']['__typename'] == 'CancelPipelineExecutionFailure'
        )
        assert result.data['cancelPipelineExecution']['message'].startswith(
            'Unable to terminate run'
        )

        context.execution_manager.terminate = old_terminate

        result = execute_dagster_graphql(
            context, RUN_CANCELLATION_QUERY, variables={'runId': run_id}
        )

        assert (
            result.data['cancelPipelineExecution']['__typename'] == 'CancelPipelineExecutionSuccess'
        )


def test_run_finished():

    instance = DagsterInstance.local_temp()
    context = define_subprocess_context(instance)
    pipeline_result = execute_pipeline(
        context.repository_definition.get_pipeline('noop_pipeline'), instance=instance
    )
    assert pipeline_result.success
    assert pipeline_result.run_id

    time.sleep(0.05)  # guarantee execution finish

    result = execute_dagster_graphql(
        context, RUN_CANCELLATION_QUERY, variables={'runId': pipeline_result.run_id}
    )

    assert result.data['cancelPipelineExecution']['__typename'] == 'CancelPipelineExecutionFailure'
    assert (
        'is not in a started state. Current status is SUCCESS'
        in result.data['cancelPipelineExecution']['message']
    )



import argparse
import sys

from setuptools import find_packages, setup


def get_version(name):
    version = {}
    with open('dagster_graphql/version.py') as fp:
        exec(fp.read(), version)  # pylint: disable=W0122

    if name == 'dagster-graphql':
        return version['__version__']
    elif name == 'dagster-graphql-nightly':
        return version['__nightly__']
    else:
        raise Exception('Shouldn\'t be here: bad package name {name}'.format(name=name))


parser = argparse.ArgumentParser()
parser.add_argument('--nightly', action='store_true')


def _do_setup(name='dagster-graphql'):
    setup(
        name='dagster_graphql',
        version=get_version(name),
        author='Elementl',
        license='Apache-2.0',
        description='The GraphQL frontend to python dagster.',
        url='https://github.com/dagster-io/dagster/tree/master/python_modules/dagster-graphql',
        classifiers=[
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: OS Independent',
        ],
        packages=find_packages(exclude=['test']),
        install_requires=[
            'graphene>=2.1.3',
            'gevent-websocket==0.10.1',
            'gevent==1.3.7',
            'requests',
        ],
        entry_points={'console_scripts': ['dagster-graphql = dagster_graphql.cli:main']},
    )


if __name__ == '__main__':
    parsed, unparsed = parser.parse_known_args()
    sys.argv = [sys.argv[0]] + unparsed
    if parsed.nightly:
        _do_setup('dagster-graphql-nightly')
    else:
        _do_setup('dagster-graphql')



import re
from collections import OrderedDict

from dagster_graphql.test.preset_query import execute_preset_query

from .setup import define_context


def test_basic_preset_query_no_presets():
    result = execute_preset_query('csv_hello_world_two', define_context())
    assert result.data == OrderedDict(
        [('pipeline', OrderedDict([('name', 'csv_hello_world_two'), ('presets', [])]))]
    )


def test_basic_preset_query_with_presets(snapshot):
    result = execute_preset_query('csv_hello_world', define_context())

    assert [preset_data['name'] for preset_data in result.data['pipeline']['presets']] == [
        'prod',
        'test',
        'test_inline',
    ]

    # Remove local filepath from snapshot
    result.data['pipeline']['presets'][2]['environmentConfigYaml'] = re.sub(
        r'num: .*/data/num.csv',
        'num: /data/num.csv',
        result.data['pipeline']['presets'][2]['environmentConfigYaml'],
    )
    snapshot.assert_match(result.data)
